#!/usr/bin/python
# vim: set et ts=4:

import sys

from helper_functions import (
    config_value_changed,
    update_config_file,
    vrs_full_restart
)

from charmhelpers.core.hookenv import (
    config,
    log,
    ERROR,
    Hooks,
    UnregisteredHookError,
    relation_get,
    status_set,
)
from charmhelpers.core.host import (
    service_stop
)
from charmhelpers.fetch import (
    apt_update, apt_install, add_source
)

hooks = Hooks()


class ConfigurationError(Exception):
    pass


@hooks.hook()
def install():
    if config('vrs-repository-url') is not None:
        key = None
        if config('vrs-ppa-key') is not None:
            key = config('vrs-ppa-key')
        add_source(config('vrs-repository-url'), key)
    else:
        e = 'vrs-repository-url is not specified'
        status_set('blocked', e)
        raise ConfigurationError(e)

    apt_update(fatal=True)
    dependencies = 'libjson-perl python-twisted-core'
    apt_install(dependencies.split(), fatal=True)


@hooks.hook('config-changed')
def config_changed():
    vrs_restart_flag = False
    if config_value_changed('vrs-packages'):
        service_stop('nuage-openvswitch-switch')
        vrs_restart_flag = True

    # Install configured packages
    apt_install(config('vrs-packages').split(), fatal=True)

    vrs_config_file = '/etc/default/openvswitch-switch'

    if (config_value_changed('vsc-controller-active') and
            config('vsc-controller-active') is not None):
        vsc_active_controller = config('vsc-controller-active')
        status_set('active',
                   'vsc_active_controller: {}'.format(vsc_active_controller))
        update_config_file(vrs_config_file, 'ACTIVE_CONTROLLER',
                           vsc_active_controller)
        vrs_restart_flag = True

    if (config_value_changed('vsc-controller-standby') and
            config('vsc-controller-standby') is not None):
        vsc_standby_controller = config('vsc-controller-standby')
        log('set vsc_standby_controller: %s' % vsc_standby_controller)
        update_config_file(vrs_config_file, 'STANDBY_CONTROLLER',
                           vsc_standby_controller)
        vrs_restart_flag = True

    if vrs_restart_flag:
        vrs_full_restart()  # Full restart to clear states


@hooks.hook('vrs-controller-service-relation-changed')
def vrs_controller_changed(relation_id=None, remote_unit=None):
    vsc_ip_address = relation_get('vsc-ip-address')
    if not vsc_ip_address:
        log('Received no vsc_ip_address in vrs_controller_changed hook')
        return


    vrs_config_file = '/etc/default/openvswitch-switch'
    update_config_file(vrs_config_file, 'ACTIVE_CONTROLLER',
                       str(vsc_ip_address))
    vrs_full_restart()
    status_set('active', 'vsc_active_controller: {}'.format(vsc_ip_address))

@hooks.hook()
def upgrade_charm():
    install()
    config_changed()


if __name__ == '__main__':
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log('Unknown hook {}, skipping'.format(e))
    except ConfigurationError as ce:
        log('Configuration error: {}'.format(ce), level=ERROR)
        sys.exit(1)
