from mock import patch, call

from test_utils import (
    CharmTestCase,
)
import hooks as hooks

with patch('charmhelpers.core.hookenv.config') as config:
    config.return_value = 'nuage-vrs'


hooks.hooks._config_save = False

TO_PATCH = [
    'config',
    'apt_update',
    'apt_install',
    'add_source',
    'relation_get',
    'service_stop',
    'update_config_file',
    'vrs_full_restart',
    'config_value_changed',
    'status_set',
]

nuage_vrs_config_changed = {
    'vrs-packages': True,
    'vsc-controller-active': True,
    'vsc-controller-standby': True,
}


def _mock_config_value_changed(option):
    return nuage_vrs_config_changed[option]


class TestNuageVRS(CharmTestCase):
    def _call_hook(self, hookname):
        hooks.hooks.execute([
            'hooks/{}'.format(hookname)])

    def setUp(self):
        super(TestNuageVRS, self).setUp(hooks, TO_PATCH)
        self.config.side_effect = self.test_config.get
        self.relation_get.side_effect = self.test_relation.get
        self.config_value_changed.side_effect = _mock_config_value_changed

    def test_install_nuage_vrs(self):
        with self.assertRaises(Exception) as context:
            self._call_hook('install')
        self.assertEqual(context.exception.message,
                         'vrs-repository-url is not specified')
        vrs_repo_url = "'http://www.nuagerepo.net/vrs'"
        self.test_config.set('vrs-repository-url', vrs_repo_url)
        _pkgs = ["libjson-perl", "python-twisted-core"]

        self._call_hook('install')
        self.add_source.assert_called_with(vrs_repo_url, None)
        self.apt_update.assert_called_with(fatal=True)
        self.apt_install.assert_has_calls([
            call(_pkgs, fatal=True),
        ])

    def test_config_changed_nuage_vrs_defaults(self):  # , config_val_changed):
        self.test_config.set('vrs-repository-url',
                             'http://www.nuagerepo.net/vrs')
        _vrspkgs = ["nuage-metadata-agent", "nuage-openvswitch-common",
                    "nuage-openvswitch-datapath-dkms",
                    "nuage-python-openvswitch", "nuage-openvswitch-switch"]

        nuage_vrs_config_changed['vrs-packages'] = False
        nuage_vrs_config_changed['vsc-controller-active'] = False
        nuage_vrs_config_changed['vsc-controller-standby'] = False
        self._call_hook('config_changed')
        self.assertFalse(self.service_stop.called)
        self.assertFalse(self.update_config_file.called)
        self.assertFalse(self.vrs_full_restart.called)
        self.apt_install.assert_has_calls([
            call(_vrspkgs, fatal=True),
        ])

        nuage_vrs_config_changed['vrs-packages'] = True
        nuage_vrs_config_changed['vsc-controller-active'] = True
        nuage_vrs_config_changed['vsc-controller-standby'] = True
        self._call_hook('config_changed')
        self.apt_install.assert_has_calls([
            call(_vrspkgs, fatal=True),
        ])
        self.assertTrue(self.service_stop.called)
        self.assertTrue(self.vrs_full_restart.called)
        self.assertFalse(self.update_config_file.called)

    def test_config_changed_nuage_vrs_active(self):  # , config_val_changed):
        self.test_config.set('vrs-repository-url',
                             'http://www.nuagerepo.net/vrs')
        _vrspkgs = ["nuage-metadata-agent", "nuage-openvswitch-common",
                    "nuage-openvswitch-datapath-dkms",
                    "nuage-python-openvswitch", "nuage-openvswitch-switch"]
        vrs_config_file = '/etc/default/openvswitch-switch'

        nuage_vrs_config_changed['vrs-packages'] = True
        nuage_vrs_config_changed['vsc-controller-active'] = True
        nuage_vrs_config_changed['vsc-controller-standby'] = True
        self.test_config.set('vsc-controller-active', '1.1.1.1')
        self._call_hook('config_changed')
        self.apt_install.assert_has_calls([
            call(_vrspkgs, fatal=True),
        ])
        self.assertTrue(self.service_stop.called)
        self.assertTrue(self.vrs_full_restart.called)
        self.update_config_file.assert_has_calls([
            call(vrs_config_file, 'ACTIVE_CONTROLLER', '1.1.1.1'),
        ])

    def test_config_changed_nuage_vrs_standby(self):  # , config_val_changed):
        self.test_config.set('vrs-repository-url',
                             'http://www.nuagerepo.net/vrs')
        _vrspkgs = ["nuage-metadata-agent", "nuage-openvswitch-common",
                    "nuage-openvswitch-datapath-dkms",
                    "nuage-python-openvswitch", "nuage-openvswitch-switch"]
        vrs_config_file = '/etc/default/openvswitch-switch'
        nuage_vrs_config_changed['vrs-packages'] = True
        nuage_vrs_config_changed['vsc-controller-active'] = True
        nuage_vrs_config_changed['vsc-controller-standby'] = True
        self.test_config.set('vsc-controller-active', '1.1.1.1')
        self.test_config.set('vsc-controller-standby', '2.2.2.2')
        self._call_hook('config_changed')
        self.apt_install.assert_has_calls([
            call(_vrspkgs, fatal=True),
        ])
        self.assertTrue(self.vrs_full_restart.called)
        self.update_config_file.assert_has_calls([
            call(vrs_config_file, 'ACTIVE_CONTROLLER', '1.1.1.1'),
            call(vrs_config_file, 'STANDBY_CONTROLLER', '2.2.2.2'),
        ])

    def test_config_changed_nuage_vrs_packages(self):  # , config_val_changed):
        self.test_config.set('vrs-repository-url',
                             'http://www.nuagerepo.net/vrs')
        _vrspkgs = ["nuage-metadata-agent", "nuage-openvswitch-common",
                    "nuage-openvswitch-datapath-dkms",
                    "nuage-python-openvswitch", "nuage-openvswitch-switch"]
        vrs_config_file = '/etc/default/openvswitch-switch'
        self.test_config.set('vsc-controller-active', '1.1.1.1')
        self.test_config.set('vsc-controller-standby', '2.2.2.2')
        nuage_vrs_config_changed['vrs-packages'] = False
        nuage_vrs_config_changed['vsc-controller-active'] = True
        nuage_vrs_config_changed['vsc-controller-standby'] = True
        self._call_hook('config_changed')
        self.apt_install.assert_has_calls([
            call(_vrspkgs, fatal=True),
        ])
        self.assertFalse(self.service_stop.called)
        self.assertTrue(self.vrs_full_restart.called)
        self.update_config_file.assert_has_calls([
            call(vrs_config_file, 'ACTIVE_CONTROLLER', '1.1.1.1'),
            call(vrs_config_file, 'STANDBY_CONTROLLER', '2.2.2.2'),
        ])

    def test_vrs_controller_relation_changed(self):
        self.test_relation.set({
            'vsc-ip-address': '10.11.12.13',
        })
        self._call_hook('vrs-controller-service-relation-changed')
        vrs_config_file = '/etc/default/openvswitch-switch'
        self.update_config_file.assert_has_calls([
            call(vrs_config_file, 'ACTIVE_CONTROLLER', '10.11.12.13'),
        ])
